package com.tenlionsoft.baselib.utils

import com.tenlionsoft.baselib.model.TimeConstants
import java.text.DateFormat
import java.text.ParseException
import java.text.SimpleDateFormat
import java.util.Calendar
import java.util.Date
import java.util.Locale

object TimeUtils {
    private val SDF_THREAD_LOCAL: ThreadLocal<SimpleDateFormat> = ThreadLocal()
    private fun getDefaultFormat(): SimpleDateFormat {
        var simpleDateFormat: SimpleDateFormat? = SDF_THREAD_LOCAL.get()
        if (simpleDateFormat == null) {
            simpleDateFormat = SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.getDefault())
            SDF_THREAD_LOCAL.set(simpleDateFormat)
        }
        return simpleDateFormat
    }

    private fun getHoursFormat(): SimpleDateFormat {
        var simpleDateFormat: SimpleDateFormat? = SDF_THREAD_LOCAL.get()
        if (simpleDateFormat == null) {
            simpleDateFormat = SimpleDateFormat("HH:mm", Locale.getDefault())
            SDF_THREAD_LOCAL.set(simpleDateFormat)
        }
        return simpleDateFormat
    }

    fun getNowDateMillis(): Long {
        return System.currentTimeMillis()
    }

    fun millis2String(millis: Long): String {
        return getDefaultFormat().format(Date(millis))
    }

    fun millis2HMStr(millis: Long): String {
        return getHoursFormat().format(Date(millis))
    }

    fun millis2String(millis: Long, format: DateFormat): String {
        return format.format(Date(millis))
    }

    fun getFriendlyTimeSpanByNow(millis: Long): String {
        val now = System.currentTimeMillis()
        val span = now - millis
        if (span < 0) return String.format("%tc", millis)
        if (span < 1000) {
            return "刚刚"
        } else if (span < TimeConstants.MIN) {
            return java.lang.String.format(Locale.getDefault(), "%d秒前", span / TimeConstants.SEC)
        } else if (span < TimeConstants.HOUR) {
            return java.lang.String.format(
                Locale.getDefault(), "%d分钟前", span / TimeConstants.MIN
            )
        }
        // 获取当天 00:00
        val wee: Long = getWeeOfToday()
        return if (millis >= wee) {
            String.format("%tR", millis)
        } else if (millis >= wee - TimeConstants.DAY) {
            String.format("昨天%tR", millis)
        } else {
            String.format("%tF", millis)
        }
    }

    private fun getWeeOfToday(): Long {
        val cal = Calendar.getInstance()
        cal[Calendar.HOUR_OF_DAY] = 0
        cal[Calendar.SECOND] = 0
        cal[Calendar.MINUTE] = 0
        cal[Calendar.MILLISECOND] = 0
        return cal.timeInMillis
    }


    /**
     * Return the friendly time span by now.
     *
     * The pattern is `yyyy-MM-dd HH:mm:ss`.
     *
     * @param time The formatted time string.
     * @return the friendly time span by now
     *
     *  * 如果小于 1 秒钟内，显示刚刚
     *  * 如果在 1 分钟内，显示 XXX秒前
     *  * 如果在 1 小时内，显示 XXX分钟前
     *  * 如果在 1 小时外的今天内，显示今天15:32
     *  * 如果是昨天的，显示昨天15:32
     *  * 其余显示，2016-10-15
     *  * 时间不合法的情况全部日期和时间信息，如星期六 十月 27 14:21:20 CST 2007
     *
     */
    fun getFriendlyTimeSpanByNow(time: String): String {
        return getFriendlyTimeSpanByNow(time, getDefaultFormat())
    }

    /**
     * Return the friendly time span by now.
     *
     * @param time   The formatted time string.
     * @param format The format.
     * @return the friendly time span by now
     *
     *  * 如果小于 1 秒钟内，显示刚刚
     *  * 如果在 1 分钟内，显示 XXX秒前
     *  * 如果在 1 小时内，显示 XXX分钟前
     *  * 如果在 1 小时外的今天内，显示今天15:32
     *  * 如果是昨天的，显示昨天15:32
     *  * 其余显示，2016-10-15
     *  * 时间不合法的情况全部日期和时间信息，如星期六 十月 27 14:21:20 CST 2007
     *
     */
    fun getFriendlyTimeSpanByNow(
        time: String, format: DateFormat
    ): String {
        return getFriendlyTimeSpanByNow(string2Millis(time, format))
    }

    fun string2Millis(time: String, format: DateFormat): Long {
        try {
            return format.parse(time)?.time ?: -1L
        } catch (e: ParseException) {
            e.printStackTrace()
        }
        return -1
    }
}